# SPDX-FileCopyrightText: Copyright (c) 2021 Art Galkin <ortemeo@gmail.com>
# SPDX-FileCopyrightText: Copyright (c) 2015 Jean-Ralph Aviles
# SPDX-License-Identifier: MIT

import unittest
from typing import List

from commie import iter_comments_ruby, Comment


def commentsToList(code:str) -> List[Comment]:
  return list(iter_comments_ruby(code))


class ShellParserTest(unittest.TestCase):

  def testComment(self):
    code = '# comment'
    comments = commentsToList(code)

    self.assertEqual(len(comments), 1)

    self.assertEqual(comments[0].code_span.extract(code), '# comment')
    self.assertEqual(comments[0].text_span.extract(code), " comment")
    self.assertEqual(comments[0].multiline, False)

  def testCommentInSingleQuotedString(self):
    code = "'this is # not a comment'"
    comments = commentsToList(code)
    self.assertEqual(comments, [])

  def testCommentInDoubleQuotedString(self):
    code = '"this is # not a comment"'
    comments = commentsToList(code)
    self.assertEqual(comments, [])

  def testNestedStringSingleOutside(self):
    code = "'this is \"# not a comment\"'"
    comments = commentsToList(code)
    self.assertEqual(comments, [])

  def testNestedStringDoubleOutside(self):
    code = '"this is \'# not a comment\'"'
    comments = commentsToList(code)
    self.assertEqual(comments, [])

  def testEscapedSingleQuote(self):
    code = "\\'# this is a comment"
    comments = commentsToList(code)

    self.assertEqual(len(comments), 1)

    self.assertEqual(comments[0].code_span.extract(code), '# this is a comment')
    self.assertEqual(comments[0].text_span.extract(code), " this is a comment")
    self.assertEqual(comments[0].multiline, False)

  def testEscapedDoubleQuote(self):
    code = '\\"# this is a comment'
    comments = commentsToList(code)

    self.assertEqual(len(comments), 1)

    self.assertEqual(comments[0].code_span.extract(code), '# this is a comment')
    self.assertEqual(comments[0].text_span.extract(code), " this is a comment")
    self.assertEqual(comments[0].multiline, False)

  def testDoubleComment(self):
    code = '# this is not # another comment'
    comments = commentsToList(code)

    self.assertEqual(len(comments), 1)

    self.assertEqual(comments[0].code_span.extract(code), '# this is not # another comment')
    self.assertEqual(comments[0].text_span.extract(code), ' this is not # another comment')
    self.assertEqual(comments[0].multiline, False)


  def testLiteralsSeparatedByComment(self):
    code = r"'This is' # 'a comment'"
    comments = commentsToList(code)

    self.assertEqual(len(comments), 1)

    self.assertEqual(comments[0].code_span.extract(code), "# 'a comment'")
    self.assertEqual(comments[0].text_span.extract(code), " 'a comment'")
    self.assertEqual(comments[0].multiline, False)



  def testDifferentLiteralsSeparatedByComment(self):
    code = r''''This is' # "a comment"'''
    comments = commentsToList(code)

    self.assertEqual(len(comments), 1)

    self.assertEqual(comments[0].code_span.extract(code), '# "a comment"')
    self.assertEqual(comments[0].text_span.extract(code), ' "a comment"')
    self.assertEqual(comments[0].multiline, False)



import unittest
from pathlib import Path
from typing import Iterable

from .parsers.c_parser import extract_comments as iter_comments_c, \
  extract_comments as iter_comments_sass
from .parsers.common import Comment, FormatUndetectedError
from .parsers.css_parser import extract_comments as iter_comments_css
from .parsers.go_parser import extract_comments as iter_comments_go
from .parsers.html_parser import extract_comments as iter_comments_html
from .parsers.js_parser import extract_comments as iter_comments_js
from .parsers.python_parser import extract_comments as iter_comments_python
from .parsers.ruby_parser import extract_comments as iter_comments_ruby
from .parsers.shell_parser import extract_comments as iter_comments_shell


def pickfunc(filename: str):
  filename = filename.lower()
  ext = filename.rpartition(".")[-1]

  if ext in ["c", "cpp", "java",
             "h", "hpp",
             # Objective-C source code 'implementation' program files usually
             # have .m filename extensions, while Objective-C 'header/interface' files
             # have .h extensions
             "m"]:
    return iter_comments_c

  if ext in ["go"]:
    return iter_comments_go

  if ext in ["js", "ts", "dart"]:
    return iter_comments_js

  if ext in ["html", "htm", "xml"]:
    return iter_comments_html

  if ext in ["rb"]:
    return iter_comments_ruby

  if ext in ["py"]:
    return iter_comments_python

  if ext in ["scss"]:
    return iter_comments_sass

  if ext in ["css"]:
    return iter_comments_css

  if ext in ["sh"]:
    return iter_comments_shell

  raise FormatUndetectedError


class TestPickFunc(unittest.TestCase):
  def test_html(self):
    self.assertEqual(pickfunc(filename="file.html"), iter_comments_html)
    self.assertEqual(pickfunc(filename="1991.HTM"), iter_comments_html)

  def test_js(self):
    self.assertEqual(pickfunc(filename="file.dart"), iter_comments_js)
    self.assertEqual(pickfunc(filename="file.js"), iter_comments_js)

  def test_undetected(self):
    with self.assertRaises(FormatUndetectedError):
      pickfunc(filename="ladeda.haha")


def iter_comments(code: str, filename: str) -> Iterable[Comment]:
  func = pickfunc(filename)
  return func(code)

def iter_comments_file(file:Path) -> Iterable[Comment]:
  return iter_comments(file.read_text(), file.name)


# SPDX-FileCopyrightText: Copyright (c) 2021 Art Galkin <ortemeo@gmail.com>
# SPDX-FileCopyrightText: Copyright (c) 2015 Jean-Ralph Aviles
# SPDX-License-Identifier: MIT

import unittest
from typing import List

from commie import iter_comments_sass
from ..parsers.common import Comment


def commentsToList(code:str) -> List[Comment]:
  return list(iter_comments_sass(code))


class SassParserTest(unittest.TestCase):

  def testSimpleMain(self):
    code = ".cssClass { /* i am\n a comment! */ }"
    comments = commentsToList(code)

    self.assertEqual(len(comments), 1)

    self.assertEqual(comments[0].code_span.extract(code), "/* i am\n a comment! */")
    self.assertEqual(comments[0].text_span.extract(code), " i am\n a comment! ")
    self.assertEqual(comments[0].multiline, True)

  def testIncLeft(self):
    code = ".cssClass { /* /* i am a comment! */ }"
    comments = commentsToList(code)

    self.assertEqual(len(comments), 1)

    self.assertEqual(comments[0].code_span.extract(code), "/* /* i am a comment! */")
    self.assertEqual(comments[0].text_span.extract(code), " /* i am a comment! ")
    self.assertEqual(comments[0].multiline, True)

  def testIncRight(self):
    code = ".cssClass { /* i am a comment! */ */ }"
    comments = commentsToList(code)

    self.assertEqual(len(comments), 1)

    self.assertEqual(comments[0].code_span.extract(code), "/* i am a comment! */")
    self.assertEqual(comments[0].text_span.extract(code), " i am a comment! ")
    self.assertEqual(comments[0].multiline, True)

  def testThreeComments(self):
    code = """
    
/* Welcome to Compass.
 * In this file you should write your main styles. (or centralize your imports)
 * Import this file using the following HTML or equivalent:
 * <link href="/stylesheets/screen.css" media="screen, projection" rel="stylesheet" type="text/css" /> */

@import "compass/reset";

@import "compass/reset";
@import "compass/css3";
@import "compass/layout";
@import "compass/utilities";
@import "compass/utilities/general/clearfix";
@import "compass/css3/pie";
@import "compass/css3/transform";


//Uses Sass' color modifying functions to adjust colors

$color1: #a9c4e5;
$color2: darken($color1, 40);
$fontcolor1: darken($color1, 80);
$fontcolor2: lighten($color1, 50);
$border-radius: 4px;


@import "basics";
    
    """
    comments = commentsToList(code)

    self.assertEqual(len(comments), 2)

    self.assertEqual(len(comments[0].code_span.extract(code).splitlines()), 4)
    self.assertEqual(len(comments[1].code_span.extract(code).splitlines()), 1)



# SPDX-FileCopyrightText: Copyright (c) 2021 Art Galkin <ortemeo@gmail.com>
# SPDX-FileCopyrightText: Copyright (c) 2015 Jean-Ralph Aviles
# SPDX-License-Identifier: MIT

from typing import NamedTuple, Optional


class Error(Exception):
  """Base Error class for all comment parsers."""


class FileError(Error):
  """Raised if there is an issue reading a file."""


class UnterminatedCommentError(Error):
  """Raised if an Unterminated multi-line comment is encountered."""

class FormatUndetectedError(Error):
  """Raised if there is an issue reading a file."""



class Span(NamedTuple):
  start: int
  end: int

  def extract(self, text: str):
    return text[self.start:self.end]


class Comment:
  """Represents comments found in a source code string."""

  def __init__(self, source:str, code_span: Span, text_span: Span, multiline: bool):

    self.source = source
    self.code_span: Span = code_span
    self.text_span: Span = text_span
    self.multiline = multiline

    self._text:Optional[str] = None
    self._markup:Optional[str] = None

  @property
  def text(self) -> str:
    if self._text is None:
      self._text = self.text_span.extract(self.source)
    return self._text

  @property
  def code(self) -> str:
    if self._text is None:
      self._text = self.code_span.extract(self.source)
    return self._text

  def __repr__(self):
    return f"Comment({self.code_span}, {self.text_span}, {self.multiline})"

  def __eq__(self, other):
    if isinstance(other, self.__class__):
      if self.__dict__ == other.__dict__:
        return True
    return False

